# 单算子API自动生成脚本

## 1 简介

单算子API自动生成脚本通过提取dump数据中的可疑算子，对其进行单API复现，输出单API精度的比对结果。具体而言，该工具可以从dump数据中提取可疑API的前反向信息，根据前反向数据生成单API的前反向过程，最后通过**新精度标准比对法**<sup>a</sup>将 NPU/GPU 和 CPU 的结果进行比对，从而给出不同比对方法下的比对结果。本工具支持**随机生成模式和真实数据模式**<sup>b</sup>。

a. 依据新精度标准，对不同的API采取不同的比对算法（包括绝对阈值法、标杆比对法、二进制一致法、ULP误差比对法和双千指标法），最终给定比对结果；

b. 在生成单API脚本时可以选择由工具构造随机数获得 dump 数据或选择真实输入的数据进行单API复现。随机生成模式（对应 task: "statistics"）执行效率高，可以快速获得结果，但数据精度低，只能大致判断精度问题；真实数据模式（对应 task: "tensor"）执行效率略低于随机生成模式，但是数据精度高，可以准确判断精度问题。

## 2 使用方式

### 前提
1. 安装 msprobe。详见[ msprobe 安装](./01.installation.md)章节。
2. 已完成对训练过程的dump，获得dump.json文件。
   [PyTorch场景的数据采集](https://gitee.com/ascend/mstt/blob/master/debug/accuracy_tools/msprobe/docs/05.data_dump_PyTorch.md)
   
   **目前仅支持复现API级的数据，故dump时level可选择L0（API信息）或者mix（module信息+API信息）。如需复现真实数据场景的API脚本，dump时task应选择tensor，如需复现随机数据场景的API脚本，dump时task选择statistics**。
3. 发现某个算子疑似存在精度问题，并得知算子名，如Functional.softmax.3、Tensor.add.0、Torch.matmul.5等

### 2.1 配置config_op.json
单API复现参数配置如下（以复现softmax算子为例）：
```
{
    "dump_json_path": "./dump.json",
    "api_name": "Functional.softmax.0",
    "extract_api_path": "./Functional.softmax.0.json",
    "propagation": "forward", 
    "data_mode": "random_data", 
    "random_seed": 1234, 
    "iter_times": 1
}
```
**配置文件参数说明**

   | 参数名称                     | 解释                                                         | 是否必选                           |
   | ---------------------------- | ------------------------------------------------------------ | ---------------------------------- |
   | dump_json_path   | dump.json的文件路径，包含所有dump算子的信息；如果已经提取了可疑算子并保存可以不指定。                                   | 否                                 |
   | api_name             | 算子名，如Functional.softmax.3、Tensor.add.0、Torch.matmul.5等。如果已经提取了可疑算子并保存可以不指定                            | 否                                 |
   | extract_api_path               | 提取可疑算子的json文件路径 | 是                                 |
   | propagation | 选择复现算子的forward还是backward，默认为forward                                                | 否                                 |
   | data_mode                 | 选择复现算子的随机数据（random_data）还是真实数据（real_data）模式，默认为random_data           | 否                                 |
   | random_seed | 仅random_data模式有效，表示手动设定的随机种子，默认为1234 | 否 |
   | iter_times             | 仅random_data模式有效，表示单API运行的次数 | 否                                 |

 ### 2.2 运行命令生成单API脚本
config_op.json配置好后，运行如下命令：
```
msprobe -f pytorch op_generate -i ./config.json -o ./
```
或者

进入到mstt的generate_op_script文件夹
```
cd mstt/debug/accuracy_tools/msprobe/pytorch/api_accuracy_checker/generate_op_script
```
运行
```
python op_generator.py -i ./config_op.json -o ./
```
**参数说明**
   | 参数名称                     | 解释                                                         | 是否必选                           |
   | ---------------------------- | ------------------------------------------------------------ | ---------------------------------- |
   | -i 或 --config_input   | config_op.json的路径                                   | 是                                 |
   | -o 或 --api_output_path             | 单API脚本的输出路径                            | 是                                 |
 
 ### 2.3 运行单API脚本
 运行完op_generator.py后，会在指定路径下生成api_name.py的单API脚本，例如Functional.softmax.3.backward.py、Tensor.add.0.forward.py、Torch.matmul.5.backward.py

运行单API脚本即可获得不同比对方法下的比对结果
```
python api_name.py
```

**运行结果参数说明**
| 字段                | 含义                                                         |
| ------------------- | ------------------------------------------------------------ |
| Shape            | 单API输出结果的shape       |
| Dtype of out_device         | NPU 或 GPU 数据的 API 数据类型。                                      |
| Dtype of out_bench        |   标杆数据的 API 数据类型。    |
| Compare Standard               |  比对方法（包括绝对阈值法，标杆比对法、二进制一致法、ULP误差比对法和双千指标法          |
| Relative Error Ratio          | 相对误差错误率。NPU 与标杆的正常值计算相对误差，其大于错误阈值的元素个数占正常值元素个数的比例。绝对阈值法指标。       |
| 相对误差判定结果         | 相对误差错误率判定结果，等于 0 标记为 pass，其余情况标记为 error。 |
| Absolute Error Ratio        | 绝对误差错误率。NPU 与标杆的小值计算绝对误差，其大于错误阈值的元素个数占小值元素个数的比例。绝对阈值法指标。NPU 或 GPU 数据与标杆数据的最大绝对误差。   |
| 绝对误差判定结果         | 绝对误差错误率判定结果，等于 0 标记为 pass，其余情况标记为 error。 |
| Small Value Error Proportion            | 小值域错误比值。NPU 与 CPU 的小值域的错误比率 / GPU 与 CPU 的小值域的错误比率。标杆比对法指标。 |
| 小值域错误判定结果       | 小值域错误比值小于等于 1 标记为 pass，1 ~ 2 之间标记为 warning，大于 2 标记为 error。 |
| Maximum Relative Error           | 相对误差最大值比值。NPU 与 CPU 的相对误差最大值 / GPU 与 CPU 的相对误差最大值。标杆比对法指标。 |
| 相对误差最大值判定结果   | 相对误差最大值比值小于等于 1 标记为 pass，1 ~ 10 之间标记为 warning，大于 10 标记为 error。 |
| Mean Relative Error            | 相对误差平均值比值。NPU 与 CPU 的相对误差的平均值 / GPU 与 CPU 的相对误差的平均值。标杆比对法指标。 |
| 相对误差平均值判定结果   | 相对误差平均值比值小于等于 1 标记为 pass，1 ~ 2 之间标记为 warning，大于 2 标记为 error。 |
| Root Mean Squared Error    | 均方根误差比值。NPU 与 CPU 的均方根误差 / GPU 与 CPU 的均方根误差。标杆比对法指标。 |
| 均方根误差判定结果       | 均方根误差比值小于等于 1 标记为 pass，1~2 之间标记为 warning，大于 2 标记为 error。 |
| Error Balance    | 误差均衡性比值。NPU 与 CPU 的误差均衡性 / GPU 与 CPU 的误差均衡性。标杆比对法指标。 |
| 误差均衡性判定结果       | 误差均衡性比值小于等于 1 标记为 pass，1 ~ 2 之间标记为 warning，大于 2 标记为 error。 |
| Error Rate    | 二进制一致错误率。NPU 或 GPU 数据中每个 Tensor 精度不一致的数值的数量与 Tensor 中数值数量的比值。只有数据是 builtin 类型（bool、int、float、str）、torch.bool 和 torch 的 int 类型或者在新精度标准中使用二进制一致算法进行比对的 API 才会展示。二进制一致法指标。 |
| 二进制一致错误率判定结果 | 二进制一致错误率判定结果，等于 0 标记为 pass，其余情况标记为 error。 |
| Maximum ULP Error    | ULP 误差最大值<sup>a</sup>。NPU 数据与标杆数据 ULP 误差的最大值（取绝对值后）。 |
| Mean ULP Error    | ULP 误差平均值<sup>a</sup>。NPU 数据与标杆数据 ULP 误差的平均值（取绝对值后）。 |
| ULP Error Proportion    |ULP 误差大于阈值占比比值<sup>a</sup>。NPU 与 CPU 的 ULP 误差大于阈值占比 / GPU 与 CPU 的 ULP 误差大于阈值占比。 |
| ULP 误差判定结果          | ULP 误差判定结果。<br/>     当 NPU 或 GPU 数据类型是 float16 或 bfloat16 时，以下两条标准满足其一标记为 pass，否则标记为 error：<br>          NPU ULP 误差大于阈值占比小于 0.001；<br/>          NPU ULP 误差大于阈值占比小于 GPU ULP 误差大于阈值占比。<br/>     当 NPU 或 GPU 数据类型是 float32 时，以下三条标准满足其一标记为 pass，否则标记为 error：<br/>          NPU ULP 误差平均值小于 64；<br/>          NPU ULP 误差大于阈值占比小于 0.05；<br/>          NPU ULP 误差大于阈值占比小于 GPU ULP 误差大于阈值占比。 |
| Thousandth ratio    |双千精度指标。是指 NPU 的 Tensor 中的元素逐个与对应的标杆数据对比，相对误差小于千分之一的个数占总元素个数的比例。测试通过标准为相对误差大于千分之一的个数占总元素个数的比例小于千分之一。仅 conv1d 和 conv2d 使用该指标。双千指标法指标。 |
| 双千指标判定结果         | 双千指标判定结果。双千指标大于 0.999 标记为 pass，否则标记为 error。 |

a：误差比对法指标。

最终判定单API是否符合精度标准由开发者通过**算子精度标准**判断。